import { Injectable } from '@angular/core';
import { HttpService } from './http.service';
import { LiveService } from './live.service';
import * as Parse from 'parse';
import { AlertController, ToastController } from '@ionic/angular';
// import AgoraRTM from 'agora-rtmClient';
declare const AgoraRTM: any;
@Injectable({
  providedIn: 'root',
})
export class MessageService {
  rtmClient: any; // RTM实例
  // rtmClientMap: any = {};
  channelNameList: any = {}; //订阅频道状态
  msChannelName: string = 'global_room'; // 全球频道
  options: any = {
    connectState: false,
  };
  appid?: string;
  userId: string = Parse.User.current()?.id!;

  pageFun?: Function; //页面传入的方法
  alert: any; // 弹窗
  constructor(
    private liveService: LiveService,
    private alertController: AlertController,
    public toastController: ToastController,
    private http: HttpService
  ) {}
  messageMapList: any = {
    global_room: [
      // 世界频道消息列表
      {
        is_self: true,
        avatar:
          'https://file-cloud.fmode.cn/Qje9D4bqol/20241109/2t1lp0032258601.png',
        msg_type: 1,
        content: 'nihao',
        create_time: new Date(),
        istoday: true,
        timeStamp: new Date(),
      },
      {
        is_self: true,
        avatar:
          'https://file-cloud.fmode.cn/Qje9D4bqol/20241109/2t1lp0032258601.png',
        msg_type: 1,
        content: `Use the pipe name to trace where the pipe is declared and used. To resolve this error: If the pipe is local to the NgModule, give it a unique name in the pipe's decorator and declared it in the NgModule. If the pipe is standalone or is declared in another NgModule, add it to the imports field of the standalone component or the current NgModule.`,
        create_time: new Date(),
        istoday: true,
        timeStamp: new Date(),
      },
      {
        is_self: true,
        avatar:
          'https://file-cloud.fmode.cn/Qje9D4bqol/20241109/2t1lp0032258601.png',
        msg_type: 1,
        content: `Use the pipe name to trace where the pipe is declared and used. To resolve this error: If the pipe is local to the NgModule, give it a unique name in the pipe's decorator and declared it in the NgModule. If the pipe is standalone or is declared in another NgModule, add it to the imports field of the standalone component or the current NgModule.`,
        create_time: new Date(),
        istoday: true,
        timeStamp: new Date(),
      },
      {
        is_self: true,
        avatar:
          'https://file-cloud.fmode.cn/Qje9D4bqol/20241109/2t1lp0032258601.png',
        msg_type: 1,
        content: `Use the pipe name to trace where the pipe is declared and used. To resolve this error: If the pipe is local to the NgModule, give it a unique name in the pipe's decorator and declared it in the NgModule. If the pipe is standalone or is declared in another NgModule, add it to the imports field of the standalone component or the current NgModule.`,
        create_time: new Date(),
        istoday: true,
        timeStamp: new Date(),
      },
      {
        is_self: false,
        avatar:
          'https://file-cloud.fmode.cn/Qje9D4bqol/20241109/2t1lp0032258601.png',
        msg_type: 1,
        content: 'nihao',
        create_time: new Date(),
        istoday: true,
        timeStamp: new Date(),
      },
    ],
  };

  /* 获取token */
  async getToken(channel?: string) {
    this.userId = Parse.User.current()?.id!;
    //获取频道token记录
    let uid = Parse.User.current()?.id;
    let baseurl = 'https://server.fmode.cn/api/webrtc/build_token';
    let reqBody = {
      company: this.liveService.company, // this.aiSer.company,
      channelName: this.msChannelName,
      type: 'withrtm',
      account: uid,
    };
    let data: any = await this.http.httpRequst(baseurl, reqBody, 'POST');
    console.log(data);
    if (data.code == 200) {
      // this.options[channel] = {
      //   token: data.data.token,
      // };
      this.options.token = data.data.token;
      this.appid = data.data.appid;
    }
  }

  async initRTM(channelName: string) {
    // let states = ['CONNECTED', 'CONNECTING'];
    if (this.options.connectState) return;
    await this.getToken(channelName);
    // const rtmConfig = { logLevel: 'debug' };
    this.rtmClient = new AgoraRTM.RTM(this.appid, this.userId);
    this.joinReady();
    await this.loginRTM();
    // this.subscribeMessage(channelName);
    // this.publishMessage('user-online', channelName, 'USER'); //用户上线通知
  }
  /**@监听频道消息
   *@'USERCALLINVITATION': 用户通话邀请
   *@'REFUSE': 拒绝
   *@'CLOASE': 拒绝
   */
  joinReady(channelName?: string) {
    this.rtmClient.addEventListener('message', async (event: any) => {
      console.log('接收到一条消息：', event);
      const message = JSON.parse(event.message);
      let is_self = event.publisher == this.userId;
      let userData = await this.getUserMetadata(event.publisher);
      if (
        event.channelName == this.userId &&
        message.text == 'USERCALLINVITATION' &&
        !is_self
      ) {
        if (this.alert) return;
        console.log(`收到${userData.nickname.value ?? '未知用户'}通话邀请`);
        this.alert = await this.alertController.create({
          cssClass: 'my-custom-class',
          header: '通话邀请',
          message: `收到${userData.nickname.value ?? '未知用户'}通话邀请`,
          buttons: [
            {
              text: '拒绝',
              role: 'cancel',
              handler: (blah) => {
                this.publishMessage('REFUSE', event.channelName);
              },
            },
            {
              text: '接受',
              cssClass: 'secondary',
              handler: () => {
                // this.sendVideoCallInvite();
              },
            },
          ],
        });
        await this.alert.present();
        return;
      } else if (
        event.channelName == this.userId &&
        message.text == 'CLOASE' &&
        !is_self
      ) {
        console.log(`${userData.nickname.value ?? '未知用户'}取消通话`);
        this.alert?.dismiss();
        const toast = await this.toastController.create({
          message: '对方已取消视频通话邀请',
          color: 'warning',
          duration: 1500,
        });
        toast.present();
        return;
      }
      this.showMessage(event);
    });
    this.rtmClient.addEventListener('presence', (event: any) => {
      console.log('频道人员状态变化 ', event);
    });
    this.rtmClient.addEventListener('linkState', (event: any) => {
      console.log('连接状态: ', event);
      // this.rtmClientMap[channelName].connectState = event.currentState;
    });
  }
  /* 加入频道 */
  // async join(channelName: string) {
  //   const options = {
  //     token: this.options[channelName].token,
  //     withPresence: true,
  //     withLock: true,
  //     withMetadata: true,
  //   };
  //   try {
  //     const streamChannel =
  //       this.rtmClientMap[channelName].rtmClient.createStreamChannel(
  //         channelName
  //       );
  //     const result = await streamChannel.join(options);
  //   } catch (status) {
  //     console.error('join channel failed: ', status);
  //   }
  // }
  async loginRTM(channelName?: string) {
    try {
      await this.rtmClient.login({
        token: this.options.token,
      });
      this.options.connectState = true; // 登录成功
      let userMateData = await this.getUserMetadata(this.userId);
      if (!userMateData) {
        let user = Parse.User.current();
        const metadata = [
          {
            key: 'nickname',
            value: user?.get('nickname'),
          },
          {
            key: 'avatar',
            value:
              user?.get('avatar') ??
              'https://file-cloud.fmode.cn/DXNgcD6zo6/20221202/j6p8kb034039.png',
          },
        ];
        const options = {
          userId: this.userId,
          addTimeStamp: true,
          addUserId: true,
        };
        let result = await this.rtmClient.storage.setUserMetadata(
          metadata,
          options
        );
        console.log(JSON.stringify(result));
      }
    } catch (status) {
      console.log(status);
    }
  }
  logOutRTM() {
    this.rtmClient.logout().then(() => {
      console.log('logout success');
      this.options.connectState = false;
    });
  }
  /* 订阅消息 */
  subscribeMessage(channelName: string, param?: any, callback?: any) {
    if (this.channelNameList[channelName]) return;
    return new Promise((resolve, reject) => {
      const options = {
        withMessage: param?.message ?? false, // message 事件
        withPresence: param?.presence ?? false, // presence 事件
        beQuiet: false, // quiet 事件
        withMetadata: false, // metadata 事件
        withLock: false, // lock 事件
      };
      this.rtmClient
        .subscribe(channelName, options)
        .then((res: any) => {
          console.log('subscribeMessage', res);
          //订阅成功
          this.channelNameList[channelName] = true;
          this.pageFun?.();
          resolve(true);
        })
        .catch((err: any) => {
          console.error('subscribeMessageErr', err);
          reject(false);
        });
    });
  }
  /* 取消订阅 */
  unsubscribeMessage(channelName: string) {
    this.rtmClient
      .unsubscribe(channelName)
      .then((res: any) => {
        console.log('unsubscribeMessage', res);
        //订阅成功
        this.channelNameList[channelName] = false;
      })
      .catch((err: any) => {
        console.error('unsubscribeMessage', err);
      });
  }

  async showMessage(param: any) {
    let userData = await this.getUserMetadata(param.publisher);
    let is_self = param.publisher == this.userId;
    console.log(userData);
    let message = JSON.parse(param.message);
    if (!this.messageMapList[param.channelName])
      this.messageMapList[param.channelName] = [];
    this.messageMapList[param.channelName].push({
      is_self: is_self,
      avatar:
        userData.avatar.value ??
        'https://file-cloud.fmode.cn/DXNgcD6zo6/20221202/j6p8kb034039.png',
      msg_type: 1,
      name: userData.nickname.value ?? '未知用户',
      content: message?.text ?? '',
      create_time: new Date(param.timeStamp),
      istoday: true,
      timeStamp: new Date(param.timeStamp),
    });
    this.pageFun?.();
  }
  async getUserMetadata(uid: string) {
    try {
      const result = await this.rtmClient.storage.getUserMetadata({
        userId: uid,
      });
      return result.metadata;
    } catch (status) {
      console.log(JSON.stringify(status));
    }
  }

  async publishMessage(
    message: string,
    channelName: string,
    channelType?: string
  ) {
    const payload = { type: 'text', text: message };
    const publishMessage = JSON.stringify(payload);
    const publishOptions = { channelType: channelType ?? 'MESSAGE' };
    try {
      const result = await this.rtmClient.publish(
        channelName,
        publishMessage,
        publishOptions
      );
      console.log(result);
    } catch (status) {
      console.log(status);
    }
  }
}
